@@ -3,11 +3,11 @@ __license__ = "Apache-2.0"
 
 # do not change this line manually
 # this is managed by git tag and updated on every release
-__version__ = '0.4.16'
+__version__ = '0.5.7'
 
 # do not change this line manually
 # this is managed by proto/build-proto.sh and updated on every execution
-__proto_version__ = '0.0.58'
+__proto_version__ = '0.0.59'
 
 import platform
 import sys
@@ -93,7 +93,7 @@ def import_classes(namespace: str, targets=None,
     :param import_once: import everything only once, to avoid repeated import
     """
 
-    import os, sys, re
+    import os, re
     from .logging import default_logger
 
     if namespace == 'jina.executors':
@@ -119,26 +119,21 @@ def import_classes(namespace: str, targets=None,
         path = os.path.dirname(pkgutil.get_loader(namespace).path)
     except AttributeError:
         if namespace == 'jina.hub':
-            default_logger.error(f'hub submodule is not initialized. Please try "git submodule update --init"')
+            default_logger.debug(f'hub submodule is not initialized. Please try "git submodule update --init"')
         return {}
 
     modules = set()
 
     for info in iter_modules([path]):
-        if not info.ispkg:
+        if (namespace != 'jina.hub' and not info.ispkg) or (namespace == 'jina.hub' and info.ispkg):
             modules.add('.'.join([namespace, info.name]))
 
     for pkg in find_packages(path):
         modules.add('.'.join([namespace, pkg]))
         pkgpath = path + '/' + pkg.replace('.', '/')
-        if sys.version_info.major == 2 or (sys.version_info.major == 3 and sys.version_info.minor < 6):
-            for _, name, ispkg in iter_modules([pkgpath]):
-                if not ispkg:
-                    modules.add('.'.join([namespace, pkg, name]))
-        else:
-            for info in iter_modules([pkgpath]):
-                if not info.ispkg:
-                    modules.add('.'.join([namespace, pkg, info.name]))
+        for info in iter_modules([pkgpath]):
+            if (namespace != 'jina.hub' and not info.ispkg) or (namespace == 'jina.hub' and info.ispkg):
+                modules.add('.'.join([namespace, pkg, info.name]))
 
     # filter
     ignored_module_pattern = r'\.tests|\.api|\.bump_version'
@@ -240,7 +235,7 @@ except ImportError:  # Windows
     res = None
 
 
-def raise_nofile(nofile_atleast=4096):
+def set_nofile(nofile_atleast=4096):
     """
     sets nofile soft limit to at least 4096, useful for running matlplotlib/seaborn on
     parallel executing plot generators vs. Ubuntu default ulimit -n 1024 or OS X El Captian 256
@@ -274,4 +269,4 @@ def raise_nofile(nofile_atleast=4096):
     return soft, hard
 
 
-raise_nofile()
+set_nofile()
