@@ -1,8 +1,10 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+if False:
+    from .python import PyClient
 
-def py_client(**kwargs):
+def py_client(**kwargs) -> 'PyClient':
     """A simple Python client for connecting to the gateway.
 
     For acceptable ``kwargs``, please refer to :command:`jina client --help`
