@@ -6,6 +6,7 @@ import base64
 from .. import __binary_delimiter__
 from ..helper import yaml
 from pkg_resources import resource_stream
+from typing import Dict, Union, List
 
 
 def _encode(clear, key=__binary_delimiter__.decode()):
@@ -27,10 +28,13 @@ def _decode(enc, key=__binary_delimiter__.decode()):
     return ''.join(dec)
 
 
-def get_default_login():
-    with resource_stream('jina', '/'.join(('resources', 'hub-builder', 'login.yml'))) as fp:
-        login_info = yaml.load(fp)
-        for k, v in login_info.items():
-            login_info[k] = _decode(v)
 
-    return login_info
+def handle_dot_in_keys(document: Dict[str, Union[Dict, List]]) -> Union[Dict, List]:
+    updated_document = {}
+    for key, value in document.items():
+        if isinstance(value, dict):
+            value = handle_dot_in_keys(value)
+        if isinstance(value, list) and len(value) > 0 and isinstance(value[0], dict):
+            value[0] = handle_dot_in_keys(value[0])
+        updated_document[key.replace('.', '_')] = value
+    return updated_document
