@@ -3,7 +3,7 @@ __license__ = "Apache-2.0"
 
 import inspect
 from functools import wraps
-from typing import Any, Dict, Callable, Tuple, Iterable, Iterator
+from typing import Any, Dict, Callable, Tuple, Iterable, Iterator, Union
 
 import ruamel.yaml.constructor
 
@@ -19,7 +19,7 @@ if False:
     import logging
 
 
-def store_init_kwargs(func):
+def store_init_kwargs(func: Callable) -> Callable:
     """Mark the args and kwargs of :func:`__init__` later to be stored via :func:`save_config` in YAML """
 
     @wraps(func)
@@ -64,7 +64,7 @@ class QuerySetReader:
     prefix.
 
     .. note::
-        - To set default value of ``arg1``, use ``self._arg1 = ``, note the underscore in the front.
+        - To set default value of ``arg1``, use ``self._arg1 =``, note the underscore in the front.
         - To access ``arg1``, simply use ``self.arg1``. It automatically switch between default ``_arg1`` and ``arg1`` from the request.
 
     For successful value reading, the following condition must be met:
@@ -93,17 +93,6 @@ class QuerySetReader:
             return self._get_parameter(name, default=self._init_kwargs_dict[name])
         raise AttributeError
 
-    # def __getattribute__(self, attr):
-    #     try:
-    #         return super().__getattribute__(attr)
-    #     except AttributeError:
-    #         if attr == '_init_kwargs_dict':
-    #             # raise attribute error to avoid recursive call
-    #             raise AttributeError
-    #         if attr in self._init_kwargs_dict:
-    #             return self._get_parameter(attr, default=self._init_kwargs_dict[name])
-    #         raise
-
 
 class DriverType(type):
 
@@ -114,8 +103,7 @@ class DriverType(type):
     @staticmethod
     def register_class(cls):
         reg_cls_set = getattr(cls, '_registered_class', set())
-        if cls.__name__ not in reg_cls_set:
-            # print('reg class: %s' % cls.__name__)
+        if cls.__name__ not in reg_cls_set or getattr(cls, 'force_register', False):
             cls.__init__ = store_init_kwargs(cls.__init__)
 
             reg_cls_set.add(cls.__name__)
@@ -185,7 +173,7 @@ class BaseDriver(metaclass=DriverType):
         raise NotImplementedError
 
     @staticmethod
-    def _dump_instance_to_yaml(data):
+    def _dump_instance_to_yaml(data) -> Dict[str, Dict]:
         # note: we only save non-default property for the sake of clarity
         a = {k: v for k, v in data._init_kwargs_dict.items()}
         r = {}
@@ -227,21 +215,27 @@ class BaseDriver(metaclass=DriverType):
 
 class BaseRecursiveDriver(BaseDriver):
 
-    def __init__(self, depth_range: Tuple[int] = (0, 1), apply_order: str = 'post',
-                 traverse_on: Tuple[str] = ('chunks',), *args, **kwargs):
+    def __init__(self,
+                 recur_on: Union[str, Tuple[str, str]] = ('chunks', 'matches'),
+                 granularity_range: Tuple[int, int] = (0, 1),
+                 adjacency_range: Tuple[int, int] = (0, 0),
+                 apply_order: str = 'post',
+                 *args,
+                 **kwargs):
         """
-
-        :param depth_range: right-exclusive range of the recursion depth, (0, 1) for root-level only
+        :param recur_on: "matches" or "chunks" or both, the actual directions where to recur on
+        :param granularity_range: right-inclusive range of the recursion depth, (0, 0) for root-level only
+        :param adjacency_range: right-inclusive range of the recursion adjacency, (0, 0) for single matches
         :param apply_order: the traverse and apply order. if 'post' then first traverse then call apply, if 'pre' then first apply then traverse
         :param args:
         :param kwargs:
         """
         super().__init__(*args, **kwargs)
-        self._depth_start = depth_range[0]
-        self._depth_end = depth_range[1]
-        if isinstance(traverse_on, str):
-            traverse_on = (traverse_on,)
-        self.traverse_fields = set(traverse_on)
+        if isinstance(recur_on, str):
+            recur_on = (recur_on, )
+        self._recur_on = recur_on
+        self._granularity_start, self._granularity_end = granularity_range
+        self._adjacency_start, self._adjacency_end = adjacency_range
         if apply_order in {'post', 'pre'}:
             self.recursion_order = apply_order
         else:
@@ -258,7 +252,7 @@ class BaseRecursiveDriver(BaseDriver):
         """
 
     def _apply_all(self, docs: Iterable['jina_pb2.Document'], context_doc: 'jina_pb2.Document', field: str, *args,
-                   **kwargs):
+                   **kwargs) -> None:
         """ Apply function works on a list of docs, modify the docs in-place
 
         Depending on the value of ``order`` of :class:`BaseRecursiveDriver`, :meth:`apply_all` applies before or after :meth:`apply`
@@ -271,42 +265,56 @@ class BaseRecursiveDriver(BaseDriver):
     def __call__(self, *args, **kwargs):
         self._traverse_apply(self.req.docs, *args, **kwargs)
 
-    def _traverse_apply(self, docs, *args, **kwargs):
+    def _traverse_apply(self, docs: Iterable['jina_pb2.Document'], *args, **kwargs) -> None:
         """often useful when you delete a recursive structure """
 
-        def post_traverse(_docs, traverse_on, context_doc=None):
+        def post_traverse(_docs, recur_on, context_doc=None,
+                          depth_name='granularity', recur_start=0, recur_end=1):
             """
             :param _docs: list of docs
-            :param traverse_on: "matches" or "chunks"
+            :param recur_on: "matches" or "chunks"
             :param context_doc: the owner of ``_docs``, if None, then it is at the very top-level
+            :param depth_name: Options are granularity or adjacency. Relates to the direction of recursion
+            :param recur_start: Starting level of recursion
+            :param recur_end: Final level of recursion
             :return:
             """
             if _docs:
                 for d in _docs:
                     # check if apply to next level
-                    if d.level_depth < self._depth_end:
-                        post_traverse(getattr(d, traverse_on), traverse_on, d)
+                    if getattr(d, depth_name) < recur_end:
+                        post_traverse(getattr(d, recur_on), recur_on, d, depth_name, recur_start, recur_end)
                     # check if apply to the current level
-                    if self._is_apply and self._depth_start <= d.level_depth < self._depth_end:
-                        self._apply(d, context_doc, traverse_on, *args, **kwargs)
+                    if self._is_apply and recur_start <= getattr(d, depth_name) < recur_end:
+                        self._apply(d, context_doc, recur_on, *args, **kwargs)
 
                 # check first doc if in the required depth range
-                if self._is_apply_all and _docs[0].level_depth >= self._depth_start:
-                    self._apply_all(_docs, context_doc, traverse_on, *args, **kwargs)
+                if self._is_apply_all and getattr(_docs[0], depth_name) >= recur_start:
+                    self._apply_all(_docs, context_doc, recur_on, *args, **kwargs)
 
-        def pre_traverse(_docs, traverse_on, context_doc=None):
+        def pre_traverse(_docs, recur_on, context_doc=None,
+                         depth_name='granularity', recur_start=0, recur_end=1):
+            """
+            :param _docs: list of docs
+            :param recur_on: "matches" or "chunks"
+            :param context_doc: the owner of ``_docs``, if None, then it is at the very top-level
+            :param depth_name: Options are granularity or adjacency. Relates to the direction of recursion
+            :param recur_start: Starting level of recursion
+            :param recur_end: Final level of recursion
+            :return:
+            """
             if _docs:
                 # check first doc if in the required depth range
-                if self._is_apply_all and _docs[0].level_depth >= self._depth_start:
-                    self._apply_all(_docs, context_doc, traverse_on, *args, **kwargs)
+                if self._is_apply_all and getattr(_docs[0], depth_name) >= recur_start:
+                    self._apply_all(_docs, context_doc, recur_on, *args, **kwargs)
 
                 for d in _docs:
                     # check if apply on the current level
-                    if self._is_apply and self._depth_start <= d.level_depth < self._depth_end:
-                        self._apply(d, context_doc, traverse_on, *args, **kwargs)
+                    if self._is_apply and recur_start <= getattr(d, depth_name) < recur_end:
+                        self._apply(d, context_doc, recur_on, *args, **kwargs)
                     # check if apply to the next level
-                    if d.level_depth < self._depth_end:
-                        pre_traverse(getattr(d, traverse_on), traverse_on, d)
+                    if getattr(d, depth_name) < recur_end:
+                        pre_traverse(getattr(d, recur_on), recur_on, d, depth_name, recur_start, recur_end)
 
         if self.recursion_order == 'post':
             _traverse = post_traverse
@@ -315,11 +323,34 @@ class BaseRecursiveDriver(BaseDriver):
         else:
             raise ValueError(f'{self.recursion_order}')
 
-        if 'chunks' in self.traverse_fields:
-            _traverse(docs, 'chunks')
-        if 'matches' in self.traverse_fields:
-            for d in docs:
-                _traverse(d.matches, 'matches')
+        if not docs:
+            return
+
+        if 'chunks' in self._recur_on:
+            working_docs = docs
+            while working_docs[0].adjacency < self._adjacency_start:
+                _temp = []
+                for working_doc in working_docs:
+                    _temp.extend(working_doc.matches)
+                working_docs = _temp
+            _traverse(working_docs, 'chunks', None, 'granularity', self._granularity_start, self._granularity_end)
+
+        if 'matches' in self._recur_on:
+            if 'chunks' in self._recur_on and self._granularity_start == 0 and self._adjacency_start == 0:
+                # Prevent double calling of _traverse for the for the full input list, since it
+                # is done already in the `if 'chunks' in self._recur_on` case.
+                if self._adjacency_end > 0:
+                    for working_doc in docs:
+                        _traverse(working_doc.matches, 'matches', None, 'adjacency', self._adjacency_start, self._adjacency_end)
+
+            else:
+                working_docs = docs
+                while working_docs[0].granularity < self._granularity_start:
+                    _temp = []
+                    for working_doc in working_docs:
+                        _temp.extend(working_doc.chunks)
+                    working_docs = _temp
+                _traverse(working_docs, 'matches', None, 'adjacency', self._adjacency_start, self._adjacency_end)
 
 
 class BaseExecutableDriver(BaseRecursiveDriver):
