@@ -29,9 +29,9 @@ class CraftDriver(BaseExecutableDriver):
                 else:
                     doc.blob.CopyFrom(array2pb(v))
             elif isinstance(protected_keys, dict) and k in protected_keys:
-                self.logger.warning(f'you are assigning a {k} in in {self.exec.__class__}, '
+                self.logger.warning(f'you are assigning a {k} in {self.exec.__class__}, '
                                     f'is it intentional? {k} will be overwritten by {self.__class__} '
-                                    f'anyway')
+                                    f'anyway.')
             elif isinstance(v, list) or isinstance(v, tuple):
                 doc.ClearField(k)
                 getattr(doc, k).extend(v)
@@ -46,7 +46,7 @@ class SegmentDriver(CraftDriver):
     def __init__(self, first_chunk_id: int = 0, random_chunk_id: bool = True,
                  level_names: List[str] = None, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        if isinstance(level_names, list) and (self._depth_end - self._depth_start + 1) != len(self.level_names):
+        if isinstance(level_names, list) and (self._granularity_end - self._granularity_start + 1) != len(self.level_names):
             self.level_names = level_names
         elif level_names is None:
             pass
@@ -56,7 +56,7 @@ class SegmentDriver(CraftDriver):
         # for adding new chunks, preorder is safer
         self.recursion_order = 'pre'
         self._counter = RandomUintCounter() if random_chunk_id else SimpleCounter(first_chunk_id)
-        self._protected_fields = {'length', 'id', 'parent_id', 'level_depth', 'mime_type'}
+        self._protected_fields = {'length', 'id', 'parent_id', 'granularity'}
 
     def _apply(self, doc: 'jina_pb2.Document', *args, **kwargs):
         _args_dict = pb_obj2dict(doc, self.exec.required_keys)
@@ -68,7 +68,9 @@ class SegmentDriver(CraftDriver):
                 c.length = len(ret)
                 c.id = next(self._counter)
                 c.parent_id = doc.id
-                c.level_depth = doc.level_depth + 1
-                c.mime_type = doc.mime_type
+                c.granularity = doc.granularity + 1
+                if not c.mime_type:
+                    c.mime_type = doc.mime_type
+
         else:
-            self.logger.warning(f'doc {doc.id} at level {doc.level_depth} gives no chunk')
+            self.logger.warning(f'doc {doc.id} at level {doc.granularity} gives no chunk')
