@@ -18,7 +18,7 @@ from ruamel.yaml import StringIO
 from .decorators import as_train_method, as_update_method, store_init_kwargs
 from .metas import get_default_metas, fill_metas_with_defaults
 from ..excepts import EmptyExecutorYAML, BadWorkspace, BadPersistantFile, NoDriverForRequest, UnattachedDriver
-from ..helper import yaml, PathImporter, expand_dict, expand_env_var, get_valid_local_config_source
+from ..helper import yaml, PathImporter, expand_dict, expand_env_var, get_local_config_source
 from ..logging.base import get_logger
 from ..logging.profile import TimeContext
 
@@ -70,8 +70,7 @@ class ExecutorType(type):
                     setattr(cls, f_name, wrapper(getattr(cls, f_name)))
 
         reg_cls_set = getattr(cls, '_registered_class', set())
-        if cls.__name__ not in reg_cls_set:
-            # print('reg class: %s' % cls.__name__)
+        if cls.__name__ not in reg_cls_set or getattr(cls, 'force_register', False):
             cls.__init__ = store_init_kwargs(cls.__init__)
             # if 'JINA_PROFILING' in os.environ:
             #     wrap_func(prof_funcs, profiling)
@@ -143,7 +142,7 @@ class BaseExecutor(metaclass=ExecutorType):
                     'on_gpu=True, but you dont have CUDA compatible GPU, i will reset on_gpu=False ')
                 self.on_gpu = False
 
-    def _post_init_wrapper(self, _metas: Dict = None, _requests: Dict = None, fill_in_metas: bool = True):
+    def _post_init_wrapper(self, _metas: Dict = None, _requests: Dict = None, fill_in_metas: bool = True) -> None:
         with TimeContext('post initiating, this may take some time', self.logger):
             if fill_in_metas:
                 if not _metas:
@@ -277,6 +276,12 @@ class BaseExecutor(metaclass=ExecutorType):
         Path(self.current_workspace).mkdir(parents=True, exist_ok=True)
         return os.path.join(self.current_workspace, name)
 
+    @property
+    def physical_size(self) -> int:
+        """Return the size of the current workspace in bytes"""
+        root_directory = Path(self.current_workspace)
+        return sum(f.stat().st_size for f in root_directory.glob('**/*') if f.is_file())
+
     def __getstate__(self):
         d = dict(self.__dict__)
         del d['logger']
@@ -296,13 +301,13 @@ class BaseExecutor(metaclass=ExecutorType):
             self.logger.warning('ImportError is often caused by a missing component, '
                                 'which often can be solved by "pip install" relevant package. %s' % ex, exc_info=True)
 
-    def train(self, *args, **kwargs):
+    def train(self, *args, **kwargs) -> None:
         """
         Train this executor, need to be overrided
         """
         pass
 
-    def touch(self):
+    def touch(self) -> None:
         """Touch the executor and change ``is_updated`` to ``True`` so that one can call :func:`save`. """
         self.is_updated = True
 
@@ -383,7 +388,7 @@ class BaseExecutor(metaclass=ExecutorType):
         :return: an executor object
         """
         if not source: raise FileNotFoundError
-        source = get_valid_local_config_source(source)
+        source = get_local_config_source(source)
         # first scan, find if external modules are specified
         with (open(source, encoding='utf8') if isinstance(source, str) else source) as fp:
             # ignore all lines start with ! because they could trigger the deserialization of that class
@@ -434,7 +439,7 @@ class BaseExecutor(metaclass=ExecutorType):
         except EOFError:
             raise BadPersistantFile(f'broken file {filename} can not be loaded')
 
-    def close(self):
+    def close(self) -> None:
         """
         Release the resources as executor is destroyed, need to be overrided
         """
@@ -524,7 +529,7 @@ class BaseExecutor(metaclass=ExecutorType):
                     return dump_path
 
     @staticmethod
-    def _dump_instance_to_yaml(data):
+    def _dump_instance_to_yaml(data) -> Dict[str, Dict]:
         # note: we only save non-default property for the sake of clarity
         _defaults = get_default_metas()
         p = {k: getattr(data, k) for k, v in _defaults.items() if getattr(data, k) != v}
