@@ -1,14 +1,17 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+import glob
 import gzip
 import json
+import shelve
 from typing import Iterator, Optional
 
 from google.protobuf.json_format import Parse
 
 from . import BaseKVIndexer
 from ... import __binary_delimiter__
+from ...helper import call_obj_fn
 from ...proto import jina_pb2
 
 
@@ -19,13 +22,6 @@ class BasePbIndexer(BaseKVIndexer):
     flush_on_add = True  #: When set to true, the indexer is flushed on every add, it is safer but slower
     mode = 't'  #: r/w mode, `t` for text, `b` for binary
 
-    def __init__(self, index_filename: str, *args, **kwargs):
-        super().__init__(index_filename, *args, **kwargs)
-
-    def post_init(self):
-        self._parser = jina_pb2.Document
-        super().post_init()
-
     def get_add_handler(self):
         """Append to the existing gzip file using text appending mode """
 
@@ -45,8 +41,8 @@ class BasePbIndexer(BaseKVIndexer):
         :param key: ``id``
         :return: protobuf chunk or protobuf document
         """
-        if self.query_handler is not None and key in self.query_handler:
-            return self.query_handler[key]
+        if self.query_handler is not None:
+            return self.query_handler.get(key, None)
 
     def get_query_handler(self):
         raise NotImplementedError
@@ -60,6 +56,50 @@ class BasePbIndexer(BaseKVIndexer):
         raise NotImplementedError
 
 
+class ShelfPbIndexer(BasePbIndexer):
+    def get_add_handler(self):
+        return shelve.open(self.index_abspath, 'w')
+
+    def get_create_handler(self):
+        return shelve.open(self.index_abspath, 'n')
+
+    def get_query_handler(self):
+        r = shelve.open(self.index_abspath, 'r')
+        self._size = len(r)
+        return r
+
+    def query(self, key: int) -> Optional['jina_pb2.Document']:
+        """ Find the protobuf chunk/doc using id
+
+        :param key: ``id``
+        :return: protobuf chunk or protobuf document
+        """
+        if self.query_handler:
+            r = self.query_handler.get(str(key), None)
+            if r:
+                d = jina_pb2.Document()
+                d.ParseFromString(r)
+                return d
+
+    def _add(self, keys: Iterator['jina_pb2.Document'], *args, **kwargs):
+        """Add a JSON-friendly object to the indexer
+
+        :param obj: an object can be jsonified
+        """
+        keys = {str(k.id): k.SerializeToString() for k in keys}
+        self._size += len(keys)
+        self.write_handler.update(keys)
+
+    def flush(self):
+        call_obj_fn(self.write_handler, 'sync')
+
+    @property
+    def is_exist(self) -> bool:
+        """Check if the database is exist or not"""
+        r = list(glob.glob(self.index_abspath + '*', recursive=True))
+        return len(r) > 0
+
+
 class JsonPbIndexer(BasePbIndexer):
     """Storing and querying protobuf chunk/document in JSON using gzip and Python dict. """
 
@@ -70,7 +110,7 @@ class JsonPbIndexer(BasePbIndexer):
                 if l:
                     tmp = json.loads(l)
                     for k, v in tmp.items():
-                        r[k] = Parse(v, self._parser())
+                        r[k] = Parse(v, jina_pb2.Document())
                         self._size += 1
         return r
 
@@ -102,7 +142,7 @@ class BinaryPbIndexer(BasePbIndexer):
 
         r = {}
         for l in tmp.split(__binary_delimiter__):
-            b = self._parser()
+            b = jina_pb2.Document()
             b.ParseFromString(l)
             r[b.id] = b
             self._size += 1
