@@ -17,7 +17,7 @@ from ruamel.yaml import StringIO
 from .. import JINA_GLOBAL
 from ..enums import FlowBuildLevel, FlowOptimizeLevel
 from ..excepts import FlowTopologyError, FlowMissingPodError, FlowBuildLevelError
-from ..helper import yaml, expand_env_var, get_non_defaults_args, deprecated_alias, random_port
+from ..helper import yaml, expand_env_var, get_non_defaults_args, deprecated_alias, random_port, complete_path
 from ..logging import get_logger
 from ..logging.sse import start_sse_logger
 from ..peapods.pod import SocketType, FlowPod, GatewayFlowPod
@@ -143,7 +143,7 @@ def _optimize_flow(op_flow, outgoing_map: Dict[str, List[str]], pod_edges: {str,
         return op_flow
 
 
-class Flow:
+class Flow(ExitStack):
     def __init__(self, args: 'argparse.Namespace' = None, **kwargs):
         """Initialize a flow object
 
@@ -164,6 +164,7 @@ class Flow:
         as the head and tail routers are removed.
         
         """
+        super().__init__()
         self.logger = get_logger(self.__class__.__name__)
         self._pod_nodes = OrderedDict()  # type: Dict[str, 'FlowPod']
         self._build_level = FlowBuildLevel.EMPTY
@@ -171,7 +172,6 @@ class Flow:
         self._last_changed_pod = ['gateway']  #: default first pod is gateway, will add when build()
 
         self._update_args(args, **kwargs)
-        self._ports_in_use = []
 
     def _update_args(self, args, **kwargs):
         from ..main.parser import set_flow_parser
@@ -263,6 +263,7 @@ class Flow:
         if not filename: raise FileNotFoundError
         if isinstance(filename, str):
             # deserialize from the yaml
+            filename = complete_path(filename)
             with open(filename, encoding='utf8') as fp:
                 return yaml.load(fp)
         else:
@@ -401,7 +402,6 @@ class Flow:
 
         kwargs.update(op_flow._common_kwargs)
         kwargs['name'] = pod_name
-        op_flow._ports_in_use += op_flow._check_port_collision(kwargs)
         op_flow._pod_nodes[pod_name] = FlowPod(kwargs=kwargs, needs=needs)
         op_flow.set_last_pod(pod_name, False)
 
@@ -462,7 +462,20 @@ class Flow:
         return self.start()
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.close()
+        super().__exit__(exc_type, exc_val, exc_tb)
+        if self.args.logserver:
+            self._stop_log_server()
+        self._build_level = FlowBuildLevel.EMPTY
+        self.logger.success(
+            f'flow is closed and all resources should be released already, current build level is {self._build_level}')
+
+    def _stop_log_server(self):
+        import urllib.request
+        try:
+            #it may have been shutdown from the outside
+            urllib.request.urlopen(JINA_GLOBAL.logserver.shutdown, timeout=5)
+        except Exception as ex:
+            self.logger.info(f'Failed to connect to shutdown log sse server: {repr(ex)}')
 
     def _start_log_server(self):
         try:
@@ -474,14 +487,15 @@ class Flow:
                                                       self.yaml_spec))
             self._sse_logger.start()
             time.sleep(1)
-            urllib.request.urlopen(JINA_GLOBAL.logserver.ready, timeout=5)
-            self.logger.success(f'logserver is started and available at {JINA_GLOBAL.logserver.address}')
+            response = urllib.request.urlopen(JINA_GLOBAL.logserver.ready, timeout=5)
+            if response.status == 200:
+                self.logger.success(f'logserver is started and available at {JINA_GLOBAL.logserver.address}')
         except ModuleNotFoundError:
             self.logger.error(
                 f'sse logserver can not start because of "flask" and "flask_cors" are missing, '
                 f'use pip install "jina[http]" (with double quotes) to install the dependencies')
-        except:
-            self.logger.error('logserver fails to start')
+        except Exception as ex:
+            self.logger.error(f'logserver fails to start: {repr(ex)}')
 
     def start(self):
         """Start to run all Pods in this Flow.
@@ -496,17 +510,13 @@ class Flow:
             self.build(copy_flow=False)
 
         if self.args.logserver:
-            self.logger.info('start logserver...')
+            self.logger.info('starting logserver...')
             self._start_log_server()
 
-        self._pod_stack = ExitStack()
         for v in self._pod_nodes.values():
-            self._pod_stack.enter_context(v)
-
-        self.logger.info('%d Pods (i.e. %d Peas) are running in this Flow' % (
-            self.num_pods,
-            self.num_peas))
+            self.enter_context(v)
 
+        self.logger.info(f'{self.num_pods} Pods (i.e. {self.num_peas} Peas) are running in this Flow')
         self.logger.success(f'flow is now ready for use, current build_level is {self._build_level}')
 
         return self
@@ -521,14 +531,6 @@ class Flow:
         """Get the number of peas (parallel count) in this flow"""
         return sum(v.num_peas for v in self._pod_nodes.values())
 
-    def close(self):
-        """Close the flow and release all resources associated to it. """
-        if hasattr(self, '_pod_stack'):
-            self._pod_stack.close()
-        self._build_level = FlowBuildLevel.EMPTY
-        self.logger.success(
-            f'flow is closed and all resources should be released already, current build level is {self._build_level}')
-
     def __eq__(self, other: 'Flow'):
         """
         Comparing the topology of a flow with another flow.
@@ -842,20 +844,5 @@ class Flow:
         """Change to use REST gateway for IO """
         self._common_kwargs['rest_api'] = True
 
-    def _check_port_collision(self, kwargs):
-        """Check if the Pods' ports collide"""
-        for _port_name in ('port_in', 'port_out', 'port_ctrl', 'port_expose'):
-            _port = kwargs.get(_port_name)
-            if _port is None:
-                _port = random_port()
-            while _port in self._ports_in_use:
-                _new = random_port()
-                _port = _new
-                self.logger.warning(f'{_port_name} collision detected. set from {_port} to {_new}')
-            kwargs[_port_name] = _port
-            self._ports_in_use.append(_port)
-            self.logger.debug(f'{kwargs.get("name")} {_port_name}: {_port}')
-        return self._ports_in_use
-
     # for backward support
     join = needs
