@@ -2,6 +2,7 @@ __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
 import functools
+import math
 import os
 import random
 import re
@@ -27,8 +28,8 @@ __all__ = ['batch_iterator', 'yaml',
            'load_contrib_module',
            'parse_arg',
            'PathImporter', 'random_port', 'get_random_identity', 'expand_env_var',
-           'colored', 'kwargs2list', 'get_valid_local_config_source', 'valid_local_config_source',
-           'cached_property']
+           'colored', 'kwargs2list', 'get_local_config_source', 'is_valid_local_config_source',
+           'cached_property', 'is_url', 'complete_path']
 
 
 def deprecated_alias(**aliases):
@@ -55,7 +56,8 @@ def rename_kwargs(func_name: str, kwargs, aliases):
             kwargs[new] = kwargs.pop(alias)
 
 
-def get_readable_size(num_bytes: int) -> str:
+def get_readable_size(num_bytes: Union[int, float]) -> str:
+    num_bytes = int(num_bytes)
     if num_bytes < 1024:
         return f'{num_bytes} Bytes'
     elif num_bytes < 1024 ** 2:
@@ -132,18 +134,28 @@ def touch_dir(base_dir: str) -> None:
         os.makedirs(base_dir)
 
 
-def batch_iterator(data: Union[Iterator[Any], List[Any], np.ndarray], batch_size: int, axis: int = 0) -> Iterator[Any]:
+def batch_iterator(data: Union[Iterator[Any], List[Any], np.ndarray], batch_size: int, axis: int = 0,
+                   yield_slice: bool = False) -> Iterator[Any]:
     if not batch_size or batch_size <= 0:
         yield data
         return
     if isinstance(data, np.ndarray):
-        if batch_size >= data.shape[axis]:
-            yield data
+        _l = data.shape[axis]
+        _d = data.ndim
+        sl = [slice(None)] * _d
+        if batch_size >= _l:
+            if yield_slice:
+                yield tuple(sl)
+            else:
+                yield data
             return
-        for _ in range(0, data.shape[axis], batch_size):
-            start = _
-            end = min(len(data), _ + batch_size)
-            yield np.take(data, range(start, end), axis, mode='clip')
+        for start in range(0, _l, batch_size):
+            end = min(_l, start + batch_size)
+            sl[axis] = slice(start, end)
+            if yield_slice:
+                yield tuple(sl)
+            else:
+                yield data[tuple(sl)]
     elif hasattr(data, '__len__'):
         if batch_size >= len(data):
             yield data
@@ -226,9 +238,11 @@ def load_contrib_module() -> List[Any]:
 class PathImporter:
 
     @staticmethod
-    def _get_module_name(absolute_path: str) -> str:
-        module_name = os.path.basename(absolute_path)
-        module_name = module_name.replace('.py', '')
+    def _get_module_name(path: str, use_abspath: bool = False, use_basename: bool = True) -> str:
+        module_name = os.path.dirname(os.path.abspath(path) if use_abspath else path)
+        if use_basename:
+            module_name = os.path.basename(module_name)
+        module_name = module_name.replace('/', '.').strip('.')
         return module_name
 
     @staticmethod
@@ -242,12 +256,18 @@ class PathImporter:
     @staticmethod
     def _path_import(absolute_path: str) -> 'types.ModuleType':
         import importlib.util
-        module_name = PathImporter._get_module_name(absolute_path)
-        spec = importlib.util.spec_from_file_location(module_name, absolute_path)
-        module = importlib.util.module_from_spec(spec)
-        spec.loader.exec_module(module)
-        sys.modules[spec.name] = module
-        return module
+        try:
+            # module_name = (PathImporter._get_module_name(absolute_path) or
+            #                PathImporter._get_module_name(absolute_path, use_abspath=True) or 'jinahub')
+
+            # I dont want to trust user path based on directory structure, "jinahub", period
+            spec = importlib.util.spec_from_file_location('jinahub', absolute_path)
+            module = importlib.util.module_from_spec(spec)
+            sys.modules[spec.name] = module  # add this line
+            spec.loader.exec_module(module)
+            return module
+        except ModuleNotFoundError:
+            pass
 
 
 _random_names = (('first', 'great', 'local', 'small', 'right', 'large', 'young', 'early', 'major', 'clear', 'black',
@@ -263,7 +283,7 @@ _random_names = (('first', 'great', 'local', 'small', 'right', 'large', 'young',
 
 
 def random_name() -> str:
-    return '-'.join(random.choice(_random_names[j]) for j in range(2))
+    return '_'.join(random.choice(_random_names[j]) for j in range(2))
 
 
 def random_port() -> int:
@@ -364,8 +384,9 @@ def expand_env_var(v: str) -> str:
         return v
 
 
-def expand_dict(d: Dict) -> Dict[str, Any]:
+def expand_dict(d: Dict, expand_fn=expand_env_var, resolve_cycle_ref=True) -> Dict[str, Any]:
     expand_map = SimpleNamespace()
+    pat = re.compile(r'{.+}|\$[a-zA-Z0-9_]*\b')
 
     def _scan(sub_d: Union[Dict, List], p):
         if isinstance(sub_d, Dict):
@@ -395,15 +416,23 @@ def expand_dict(d: Dict) -> Dict[str, Any]:
                 if isinstance(v, dict) or isinstance(v, list):
                     _replace(v, p.__dict__[k])
                 else:
-                    if isinstance(v, str) and (re.match(r'{.*?}', v) or re.match(r'\$.*\b', v)):
-                        sub_d[k] = expand_env_var(v.format(root=expand_map, this=p))
+                    if isinstance(v, str) and pat.findall(v):
+                        sub_d[k] = _sub(v, p)
         elif isinstance(sub_d, List):
             for idx, v in enumerate(sub_d):
                 if isinstance(v, dict) or isinstance(v, list):
                     _replace(v, p[idx])
                 else:
-                    if isinstance(v, str) and (re.match(r'{.*?}', v) or re.match(r'\$.*\b', v)):
-                        sub_d[idx] = expand_env_var(v.format(root=expand_map, this=p))
+                    if isinstance(v, str) and pat.findall(v):
+                        sub_d[idx] = _sub(v, p)
+
+    def _sub(v, p):
+        if resolve_cycle_ref:
+            try:
+                v = v.format(root=expand_map, this=p)
+            except KeyError:
+                pass
+        return expand_fn(v)
 
     _scan(d, expand_map)
     _replace(d, expand_map)
@@ -535,18 +564,16 @@ def kwargs2list(kwargs: Dict) -> List[str]:
     return args
 
 
-def get_valid_local_config_source(path: str, to_stream: bool = False) -> Union[StringIO, TextIO, str]:
+def get_local_config_source(path: str, to_stream: bool = False) -> Union[StringIO, TextIO, str]:
     # priority, filepath > classname > default
     import io
     from pkg_resources import resource_filename
     if hasattr(path, 'read'):
         # already a readable stream
         return path
-    elif os.path.exists(path):
-        if to_stream:
-            return open(path, encoding='utf8')
-        else:
-            return path
+    elif path.endswith('.yml') or path.endswith('.yaml'):
+        _p = complete_path(path)
+        return open(_p, encoding='utf8') if to_stream else _p
     elif path.startswith('_') and os.path.exists(
             resource_filename('jina', '/'.join(('resources', 'executors.%s.yml' % path)))):
         return resource_filename('jina', '/'.join(('resources', 'executors.%s.yml' % path)))
@@ -566,15 +593,15 @@ def get_valid_local_config_source(path: str, to_stream: bool = False) -> Union[S
         # possible class name
         return io.StringIO(f'!{path}')
     else:
-        raise FileNotFoundError('%s can not be resolved, it should be a readable stream,'
-                                ' or a valid file path, or a supported class name.' % path)
+        raise FileNotFoundError(f'{path} can not be resolved, it should be a readable stream,'
+                                ' or a valid file path, or a supported class name.')
 
 
-def valid_local_config_source(path: str) -> bool:
+def is_valid_local_config_source(path: str) -> bool:
     try:
-        get_valid_local_config_source(path)
+        get_local_config_source(path)
         return True
-    except:
+    except FileNotFoundError:
         return False
 
 
@@ -693,3 +720,47 @@ class cached_property:
 def get_now_timestamp():
     now = datetime.now()
     return int(datetime.timestamp(now))
+
+
+def complete_path(path: str) -> str:
+    import inspect
+    _p = None
+
+    if os.path.exists(path):
+        # this checks both abs and relative paths already
+        _p = path
+    else:
+        search_paths = []
+        frame = inspect.currentframe()
+
+        # iterates on whoever calls me
+        while frame:
+            search_paths.append(os.path.dirname(inspect.getfile(frame)))
+            frame = frame.f_back
+        search_paths += os.environ['PATH'].split(os.pathsep)
+
+        # not in local path, search within all search paths
+        for p in search_paths:
+            _p = os.path.join(p, path)
+            if os.path.exists(_p):
+                break
+    if _p:
+        return _p
+    else:
+        raise FileNotFoundError(f'can not find {path}')
+
+
+def get_readable_time(*args, **kwargs):
+    import datetime
+    secs = float(datetime.timedelta(*args, **kwargs).total_seconds())
+    units = [('day', 86400), ('hour', 3600), ('minute', 60), ('second', 1)]
+    parts = []
+    for unit, mul in units:
+        if secs / mul >= 1 or mul == 1:
+            if mul > 1:
+                n = int(math.floor(secs / mul))
+                secs -= n * mul
+            else:
+                n = int(secs)
+            parts.append(f'{n} {unit}' + ('' if n == 1 else 's'))
+    return ' and '.join(parts)
