@@ -39,29 +39,31 @@ ac_table = {
                 '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                 '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--replica-id',
                 '--check-version', '--array-in-pb', '--compress-hwm', '--compress-lwm', '--num-part', '--role',
-                '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--log-sse', '--log-remote',
-                '--log-profile', '--log-with-own-name', '--host', '--port-expose', '--port-grpc', '--max-message-size',
-                '--proxy', '--parallel', '--shards', '--polling', '--scheduling', '--uses-reducing', '--shutdown-idle'],
-        'flow': ['--help', '--yaml-path', '--logserver', '--logserver-config', '--optimize-level', '--output-type',
+                '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-sse',
+                '--log-remote', '--log-profile', '--log-with-own-name', '--host', '--port-expose', '--port-grpc',
+                '--max-message-size', '--proxy', '--parallel', '--shards', '--polling', '--scheduling', '--uses-before',
+                '--uses-after', '--shutdown-idle'],
+        'flow': ['--help', '--uses', '--logserver', '--logserver-config', '--optimize-level', '--output-type',
                  '--output-path'],
         'gateway': ['--help', '--name', '--identity', '--uses', '--py-modules', '--uses-internal', '--entrypoint',
                     '--pull-latest', '--volumes', '--port-in', '--port-out', '--host-in', '--host-out', '--socket-in',
                     '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                     '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--replica-id',
                     '--check-version', '--array-in-pb', '--compress-hwm', '--compress-lwm', '--num-part', '--role',
-                    '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--log-sse', '--log-remote',
-                    '--log-profile', '--log-with-own-name', '--host', '--port-expose', '--port-grpc',
+                    '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-sse',
+                    '--log-remote', '--log-profile', '--log-with-own-name', '--host', '--port-expose', '--port-grpc',
                     '--max-message-size', '--proxy', '--prefetch', '--prefetch-on-recv', '--allow-spawn', '--rest-api'],
         'ping': ['--help', '--timeout', '--retries', '--print-response'],
-        'check': ['--help', '--summary-exec', '--summary-driver'], 'hub': ['--help', 'build', 'push', 'pull'],
+        'check': ['--help', '--summary-exec', '--summary-driver'],
+        'hub': ['--help', 'new', 'init', 'create', 'build', 'push', 'pull'],
         'pea': ['--help', '--name', '--identity', '--uses', '--py-modules', '--uses-internal', '--entrypoint',
                 '--pull-latest', '--volumes', '--port-in', '--port-out', '--host-in', '--host-out', '--socket-in',
                 '--socket-out', '--port-ctrl', '--ctrl-with-ipc', '--timeout', '--timeout-ctrl', '--timeout-ready',
                 '--dump-interval', '--exit-no-dump', '--read-only', '--separated-workspace', '--replica-id',
                 '--check-version', '--array-in-pb', '--compress-hwm', '--compress-lwm', '--num-part', '--role',
-                '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--log-sse', '--log-remote',
-                '--log-profile', '--log-with-own-name', '--host', '--port-expose', '--port-grpc', '--max-message-size',
-                '--proxy'], 'log': ['--help', '--groupby-regex', '--refresh-time'],
+                '--skip-on-error', '--memory-hwm', '--runtime', '--max-idle-time', '--daemon', '--log-sse',
+                '--log-remote', '--log-profile', '--log-with-own-name', '--host', '--port-expose', '--port-grpc',
+                '--max-message-size', '--proxy'], 'log': ['--help', '--groupby-regex', '--refresh-time'],
         'client': ['--help', '--host', '--port-expose', '--port-grpc', '--max-message-size', '--proxy', '--batch-size',
                    '--mode', '--top-k', '--mime-type', '--callback-on-body', '--first-request-id', '--first-doc-id',
                    '--random-doc-id', '--timeout-ready', '--filter-by', '--skip-dry-run'],
