@@ -7,6 +7,7 @@ import os
 import threading
 import time
 import traceback
+from multiprocessing.synchronize import Event
 from queue import Empty
 from typing import Dict, Optional, Union
 
@@ -16,13 +17,13 @@ from .zmq import send_ctrl_message, Zmqlet, ZmqStreamlet
 from .. import __ready_msg__, __stop_msg__
 from ..drivers.helper import routes2str, add_route
 from ..enums import PeaRoleType, OnErrorSkip
-from ..excepts import NoExplicitMessage, ExecutorFailToLoad, MemoryOverHighWatermark, DriverError
+from ..excepts import NoExplicitMessage, ExecutorFailToLoad, MemoryOverHighWatermark, DriverError, PeaFailToStart
 from ..executors import BaseExecutor
+from ..helper import is_valid_local_config_source
 from ..logging import get_logger
 from ..logging.profile import used_memory, TimeDict
 from ..proto import jina_pb2
-from ..helper import valid_local_config_source
-from multiprocessing.synchronize import Event
+from ..logging.queue import clear_queues
 
 __all__ = ['PeaMeta', 'BasePea']
 
@@ -110,7 +111,7 @@ class BasePea(metaclass=PeaMeta):
         super().__init__()
         self.args = args
         self.name = self.__class__.__name__  #: this is the process name
-        self.daemon = True
+        self.daemon = args.daemon
 
         self.is_ready = _get_event(self)
         self.is_shutdown = _get_event(self)
@@ -125,22 +126,22 @@ class BasePea(metaclass=PeaMeta):
         self._request = None
         self._message = None
 
-        if isinstance(args, argparse.Namespace):
-            if args.name:
-                self.name = args.name
-            if args.role == PeaRoleType.REPLICA:
-                self.name = '%s-%d' % (self.name, args.replica_id)
-            self.ctrl_addr, self.ctrl_with_ipc = Zmqlet.get_ctrl_address(args)
-            if not args.log_with_own_name and args.name:
+        if isinstance(self.args, argparse.Namespace):
+            if self.args.name:
+                self.name = self.args.name
+            if self.args.role == PeaRoleType.REPLICA:
+                self.name = '%s-%d' % (self.name, self.args.replica_id)
+            self.ctrl_addr, self.ctrl_with_ipc = Zmqlet.get_ctrl_address(self.args)
+            if not self.args.log_with_own_name and self.args.name:
                 # everything in this Pea (process) will use the same name for display the log
-                os.environ['JINA_POD_NAME'] = args.name
-            self.logger = get_logger(self.name, **vars(args))
+                os.environ['JINA_POD_NAME'] = self.args.name
+            self.logger = get_logger(self.name, **vars(self.args))
         else:
             self.logger = get_logger(self.name)
 
     def __str__(self):
         r = self.name
-        if self.executor is not None:
+        if getattr(self, 'executor', None):
             r += f'({str(self.executor)})'
         return r
 
@@ -190,8 +191,9 @@ class BasePea(metaclass=PeaMeta):
         """
         if self.args.uses:
             try:
-                self.executor = BaseExecutor.load_config(self.args.uses if valid_local_config_source(self.args.uses) else self.args.uses_internal,
-                                                         self.args.separated_workspace, self.args.replica_id)
+                self.executor = BaseExecutor.load_config(
+                    self.args.uses if is_valid_local_config_source(self.args.uses) else self.args.uses_internal,
+                    self.args.separated_workspace, self.args.replica_id)
                 self.executor.attach(pea=self)
             except FileNotFoundError:
                 raise ExecutorFailToLoad
@@ -257,6 +259,21 @@ class BasePea(metaclass=PeaMeta):
             self.pre_hook(msg).handle(msg).post_hook(msg)
         return msg
 
+    def _handle_terminate_signal(self, msg):
+        # save executor
+        if hasattr(self, 'executor'):
+            if not self.args.exit_no_dump:
+                self.save_executor(dump_interval=0)
+            self.executor.close()
+
+        # serious error happen in callback, we need to break the event loop
+        self.zmqlet.send_message(msg)
+        # note, the logger can only be put on the second last line before `close`, as when
+        # `close` is called, the callback is unregistered and everything after `close` can not be reached
+        # some black magic in eventloop i guess?
+        self.loop_teardown()
+        self.is_shutdown.set()
+
     def msg_callback(self, msg: 'jina_pb2.Message') -> Optional['jina_pb2.Message']:
         """Callback function after receiving the message
 
@@ -267,14 +284,9 @@ class BasePea(metaclass=PeaMeta):
             # generally unless executor throws an OSError, the exception are caught and solved inplace
             self.zmqlet.send_message(self._callback(msg))
         except (SystemError, zmq.error.ZMQError, KeyboardInterrupt) as ex:
-            # serious error happen in callback, we need to break the event loop
-            self.zmqlet.send_message(msg)
-            # note, the logger can only be put on the second last line before `close`, as when
-            # `close` is called, the callback is unregistered and everything after `close` can not be reached
-            # some black magic in eventloop i guess?
+            # save executor
             self.logger.info(f'{repr(ex)} causes the breaking from the event loop')
-
-            self.zmqlet.close()
+            self._handle_terminate_signal(msg)
         except MemoryOverHighWatermark:
             self.logger.critical(
                 f'memory usage {used_memory()} GB is above the high-watermark: {self.args.memory_hwm} GB')
@@ -319,10 +331,6 @@ class BasePea(metaclass=PeaMeta):
 
     def loop_teardown(self):
         """Stop the request loop """
-        if hasattr(self, 'executor'):
-            if not self.args.exit_no_dump:
-                self.save_executor(dump_interval=0)
-            self.executor.close()
         if hasattr(self, 'zmqlet'):
             self.zmqlet.close()
 
@@ -332,9 +340,11 @@ class BasePea(metaclass=PeaMeta):
             self.post_init()
             self.loop_body()
         except ExecutorFailToLoad:
-            self.logger.critical(f'can not start a executor from {self.args.uses}')
-        except (SystemError, zmq.error.ZMQError, KeyboardInterrupt):
-            pass
+            self.logger.critical(f'can not start a executor from {self.args.uses}', exc_info=True)
+        except KeyboardInterrupt:
+            self.logger.info('Loop interrupted by user')
+        except SystemError as ex:
+            self.logger.error(f'SystemError interrupted pea loop {repr(ex)}')
         except DriverError as ex:
             self.logger.critical(f'driver error: {repr(ex)}', exc_info=True)
         except zmq.error.ZMQError:
@@ -345,6 +355,7 @@ class BasePea(metaclass=PeaMeta):
             # - self.zmqlet.send_message
             self.logger.critical(f'unknown exception: {repr(ex)}', exc_info=True)
         finally:
+            # if an exception occurs this unsets ready and shutting down
             self.loop_teardown()
             self.unset_ready()
             self.is_shutdown.set()
@@ -361,7 +372,7 @@ class BasePea(metaclass=PeaMeta):
         """
         pass
 
-    def close(self) -> None:
+    def send_terminate_signal(self) -> None:
         """Gracefully close this pea and release all resources """
         if self.is_ready.is_set() and hasattr(self, 'ctrl_addr'):
             return send_ctrl_message(self.ctrl_addr, jina_pb2.Request.ControlRequest.TERMINATE,
@@ -388,7 +399,10 @@ class BasePea(metaclass=PeaMeta):
 
         if self.ready_or_shutdown.wait(_timeout):
             if self.is_shutdown.is_set():
-                self.logger.critical(f'fail to start {self.__class__} with name {self.name}')
+                # return too early and the shutdown is set, means something fails!!
+                self.logger.critical(f'fail to start {self.__class__} with name {self.name}, '
+                                     f'this often means the executor used in the pod is not valid')
+                raise PeaFailToStart
             return self
         else:
             raise TimeoutError(
@@ -397,5 +411,12 @@ class BasePea(metaclass=PeaMeta):
     def __enter__(self) -> 'BasePea':
         return self.start()
 
+    def close(self) -> None:
+        self.send_terminate_signal()
+        self.is_shutdown.wait()
+        if not self.daemon:
+            clear_queues()
+            self.join()
+
     def __exit__(self, exc_type, exc_val, exc_tb) -> None:
         self.close()
